package com.caimei.controller;

import com.caimei.model.ResponseJson;
import com.caimei.model.vo.LogisticsBatchVo;
import com.caimei.model.vo.OrderProductVo;
import com.caimei.model.vo.OrderVo;
import com.caimei.service.OrderService;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

/**
 * Description
 *
 * @author : plf
 * @date : 2021/3/26
 */
@Api(tags = "订单相关")
@RestController
@RequiredArgsConstructor
@RequestMapping("/order")
public class OrderApi {
    private final OrderService orderService;

    /**
     * 我的订单
     */
    @ApiOperation("我的订单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderState", required = true, value = "1:待付款,2:待发货,3:已发货,4:退货款"),
            @ApiImplicitParam(name = "userId", required = false, value = "用户id"),
            @ApiImplicitParam(name = "pageNum", required = false, value = "第几页"),
            @ApiImplicitParam(name = "pageSize", required = false, value = "一页多少条")
    })
    @GetMapping("/list")
    public ResponseJson<PageInfo<OrderVo>> getOrderList(Integer orderState, Integer userId,
                                                        @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                                                        @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize) {
        return orderService.getOrderList(orderState, userId, pageNum, pageSize);
    }

    /**
     * 订单详情
     */
    @ApiOperation("订单详情")
    @ApiImplicitParam(name = "orderId", required = true, value = "订单id")
    @GetMapping("/detail")
    public ResponseJson<Map<String, Object>> getOrderDetail(Integer orderId) {
        if (orderId == null) {
            return ResponseJson.error("参数异常", null);
        }
        return orderService.getOrderDetail(orderId);
    }

    /**
     * 查看物流
     */
    @ApiOperation("查看物流")
    @ApiImplicitParam(name = "orderId", required = true, value = "订单id")
    @GetMapping("/logistics")
    public ResponseJson<List<LogisticsBatchVo>> logistics(Integer orderId) {
        if (orderId == null) {
            return ResponseJson.error("参数异常", null);
        }
        return orderService.logistics(orderId);
    }

    /**
     * 确认收货
     */
    @ApiOperation("确认收货")
    @ApiImplicitParam(name = "orderId", required = true, value = "订单id")
    @GetMapping("/affirm")
    public ResponseJson<String> affirmCargo(Integer orderId) {
        return orderService.affirmCargo(orderId);
    }

    /**
     * 删除订单
     */
    @GetMapping("/delete")
    @ApiOperation("删除订单")
    @ApiImplicitParam(name = "orderId", required = true, value = "订单id")
    public ResponseJson<String> deleteOrder(Integer orderId) {
        return orderService.deleteOrder(orderId);
    }

    /**
     * 取消订单
     */
    @ApiOperation("取消订单")
    @ApiImplicitParam(name = "orderId", required = true, value = "订单id")
    @GetMapping("/cancel")
    public ResponseJson<String> cancelOrder(Integer orderId) {
        return orderService.cancelOrder(orderId);
    }

    /**
     * 根据关键词获取订单
     */
    @ApiOperation("根据关键词获取订单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "searchWord", required = true, value = "搜索关键词"),
            @ApiImplicitParam(name = "userId", required = true, value = "用户id"),
            @ApiImplicitParam(name = "pageNum", required = false, value = "第几页"),
            @ApiImplicitParam(name = "pageSize", required = false, value = "一页多少条")
    })
    @GetMapping("/search")
    public ResponseJson<PageInfo<OrderVo>> getProductsBySearch(String searchWord, Integer userId,
                                                               @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                                                               @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize) {
        if (StringUtils.isBlank(searchWord)) {
            return ResponseJson.error("请输入搜索关键词", null);
        }
        if (userId == null) {
            return ResponseJson.error("用户参数异常", null);
        }
        return orderService.searchOrder(searchWord, userId, pageNum, pageSize);
    }

    /**
     * 根据组织ID查找历史记录
     */
    @ApiOperation("根据组织ID查找历史记录")
    @ApiImplicitParam(name = "userId", required = true, value = "用户id")
    @GetMapping("/searchHistory")
    public ResponseJson<List<String>> getSearchHistory(Integer userId) {
        return orderService.getSearchHistory(userId);
    }

    /**
     * 根据组织ID删除历史记录
     */
    @ApiOperation("根据组织ID删除历史记录")
    @ApiImplicitParam(name = "userId", required = true, value = "用户id")
    @GetMapping("/searchHistory/delete")
    public ResponseJson<String> deleteSearchHistory(Integer userId) {
        return orderService.deleteSearchHistory(userId);
    }

    /**
     * 订单数量统计
     */
    @ApiOperation("订单数量统计")
    @ApiImplicitParam(name = "userId", required = true, value = "用户id")
    @GetMapping("/order/total")
    public ResponseJson<Map<String, Object>> orderTotal(Integer userId) {
        return orderService.orderTotal(userId);
    }


    /**
     * 分销者订单列表
     */
    @ApiOperation("分销者订单列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderState", required = false, value = "1:待付款,2:待发货,3:已发货,4:退货款"),
            @ApiImplicitParam(name = "orderType", required = true, value = "0:全部订单,1:用户订单,2:自身订单"),
            @ApiImplicitParam(name = "userId", required = true, value = "分销者用户id"),
            @ApiImplicitParam(name = "searchWord", required = false, value = "搜索关键词"),
            @ApiImplicitParam(name = "pageNum", required = false, value = "第几页"),
            @ApiImplicitParam(name = "pageSize", required = false, value = "一页多少条")
    })
    @GetMapping("/dealer/list")
    public ResponseJson<PageInfo<OrderVo>> dealerOrderList(Integer orderState, Integer orderType, Integer userId, String searchWord,
                                                           @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                                                           @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize) {
        if (userId == null) {
            return ResponseJson.error("用户参数异常", null);
        }
        return orderService.dealerOrderList(orderState, orderType, userId, searchWord, pageNum, pageSize);
    }

    /**
     * 再次购买
     */
    @ApiOperation("再次购买")
    @ApiImplicitParam(name = "orderId", required = true, value = "订单id")
    @GetMapping("/buyAgain")
    public ResponseJson<List<OrderProductVo>> buyAgain(Integer orderId) {
        if (orderId == null) {
            return ResponseJson.error("参数异常", null);
        }
        return orderService.buyAgain(orderId);
    }
}
